@@ -21,22 +21,21 @@
 #'
 #' @title ANFIS updating function
 #'
-#' @param data.train a matrix(m x n) of normalized data for the training process, where m is the number of instances and 
-#' n is the number of variables; the last column is the output variable.
-#' @param range.input the range of the input variables, as a matrix(2 x n).
-#' @param range.output the range of the output variable, as a matrix(2 x n).
+#' @param data.train a matrix (\eqn{m \times n}) of normalized data for the training process, where \eqn{m} is the number of instances and 
+#' \eqn{n} is the number of variables; the last column is the output variable.
+#' @param def a predicted value
 #' @param rule.data.num a matrix containing the rule base in integer form. 
 #' @param miu.rule a matrix with the degrees of rules. See \code{\link{inference}}.
 #' @param func.tsk a matrix of parameters of the function on the consequent part using the Takagi Sugeno Kang model.
 #' @param varinp.mf a matrix of parameters of membership functions of the input variables.
 #' @param step.size a real number between 0 and 1 representing the step size of 
 #' the gradient descent. 
-#' @seealso \code{\link{ANFIS}}, \code{\link{rulebase}}, \code{\link{inference}}, \code{\link{fuzzifier}} and \code{\link{defuzzifier}}
+# @seealso \code{\link{ANFIS}}, \code{\link{rulebase}}, \code{\link{inference}}, \code{\link{fuzzifier}} and \code{\link{defuzzifier}}
 # @return A list containing the following components:
-# \item{func.tsk.new}{a matrix of parameters of the consequence part, if using Takagi Sugeno Kang model}
-# \item{varinp.mf}{a matrix to generate the shapes of the membership functions on the input variables}
+# \item{\code{func.tsk.new}}{a matrix of parameters of the consequence part, if using Takagi Sugeno Kang model}
+# \item{\code{varinp.mf}}{a matrix to generate the shapes of the membership functions on the input variables}
 # @export
-ANFIS.update <- function(data.train, range.input, range.output, rule.data.num, miu.rule, func.tsk, varinp.mf, step.size = 0.01){
+ANFIS.update <- function(data.train, def, rule.data.num, miu.rule, func.tsk, varinp.mf, step.size = 0.01){
 
 ## for now, we use single output
 num.varoutput <- 1 
@@ -45,34 +44,12 @@ alpha <- step.size
 
 ## set data into matrix with number of column 1
 data.m <-matrix(data[1, 1 : (ncol(data.train) - num.varoutput)], ncol = 1)
+miu.rule.t <- as.matrix(miu.rule)
 
 ## split linear equation on consequent part	
 func.tsk.var <- func.tsk[, 1: ncol(func.tsk) - 1]
 func.tsk.cont <- func.tsk[, ncol(func.tsk), drop = FALSE]
 
-## calculate linear eq.	
-ff <- func.tsk.var %*% data.m + func.tsk.cont
-
-## set into matrix
-miu.rule.t <- as.matrix(miu.rule)
-
-## get nominator on consequent part
-cum <- sum(ff * t(miu.rule.t))
-
-## get denominator on consequent part	
-div <- sum(miu.rule.t)
-
-## set if divide by zero and calculate predicted value "def"
-if (div == 0)
-	def <- 0
-else {
-	def <- cum / div
-	if (def > max(range.output, na.rm=TRUE))
-		def <- max(range.output, na.rm=TRUE)
-	else if (def < min(range.input, na.rm=TRUE))
-		def <- min(range.input, na.rm=TRUE)
-	}
-
 ## find max value on miu.rule
 indxx <- which.max(miu.rule.t)
 
@@ -82,26 +59,28 @@ gal <- (def - data.train[1, ncol(data.train)])
 ## initialize 
 delta.out <- 0
 
-## update on consequent part, TSK model
-	for (m in 1 : nrow(func.tsk.var)){
-		for (n in 1 : ncol(func.tsk.var)){
-			sum.miu <- sum(miu.rule.t, na.rm = TRUE)
-			if (sum.miu != 0)
-				## update consequent part
-				func.tsk.var[m, n] <- func.tsk.var[m, n] - alpha * gal * (miu.rule[m]/sum.miu) *  data.train[1, n]	
-			else
-				func.tsk.var[m, n] <- func.tsk.var[m, n] - alpha * gal * (miu.rule[m]) *  data.train[1, n]	
-		}	
+## vectorize the function
+sum.miu <- sum(miu.rule.t, na.rm = TRUE)
+function.tsk <- function(i, j, func.tsk.var){	
+	if (sum.miu != 0) {
+			## update consequent part
+			func.tsk.var[i, j] <<- func.tsk.var[i, j] - alpha * gal * (miu.rule[i]/sum.miu) *  data.train[1, j]	
+	} else {
+			func.tsk.var[i, j] <<- func.tsk.var[i, j] - alpha * gal * (miu.rule[i]) *  data.train[1, j]	
+	}	
+}	
+vec.func.tsk <- Vectorize(function.tsk, vectorize.args=list("i","j"))
+outer(1 : nrow(func.tsk.var), 1 : ncol(func.tsk.var), vec.func.tsk, func.tsk.var)
+
+
+for (mm in 1 : nrow(func.tsk.cont)){
+	if (sum.miu != 0) {
+		## update constant on linear eq of consequent part
+		func.tsk.cont[mm, 1] <- func.tsk.cont[mm, 1] - alpha * gal * (miu.rule[mm]/sum.miu)
+	} else {
+		func.tsk.cont[mm, 1] <- func.tsk.cont[mm, 1] - alpha * gal * miu.rule[mm]
 	}
-	
-	for (mm in 1 : nrow(func.tsk.cont)){
-		sum.miu <- sum(miu.rule.t, na.rm = TRUE)
-		if (sum.miu != 0)
-			## update constant on linear eq of consequent part
-			func.tsk.cont[mm, 1] <- func.tsk.cont[mm, 1] - alpha * gal * (miu.rule[mm]/sum.miu)
-		else
-			func.tsk.cont[mm, 1] <- func.tsk.cont[mm, 1] - alpha * gal * miu.rule[mm]
- 	}
+}
 
 ## collect new linear eq. 
 func.tsk.new <- cbind(func.tsk.var, func.tsk.cont)
@@ -117,17 +96,17 @@ num.varinp <- ncol(data.train) - num.varoutput
 for (ii in 1 : ncol(miu.rule.t)) {
 		
 	## split linear eq.
-	func.tsk.var.indxx <- func.tsk[ii, 1: ncol(func.tsk) - 1]
-	func.tsk.cont.indxx <- func.tsk[ii, ncol(func.tsk)]
+	func.tsk.var.indxx <- func.tsk.new[ii, 1: ncol(func.tsk.new) - 1]
+	func.tsk.cont.indxx <- func.tsk.new[ii, ncol(func.tsk.new)]
 	
 	## calculate predicted value
 	ff.indxx <- func.tsk.var.indxx %*% data.m + func.tsk.cont.indxx
 	tau.miu <- ff.indxx * miu.rule.t[1, ii]
+	div <- sum(miu.rule.t)
 	if (div == 0 )
 		div <- 0.0001
 	func.val <- tau.miu / div
-	
-	
+		
 	num.varinput <- ncol(data.train) - num.varoutput
 	
 	## update parameters in antecedent part
@@ -153,7 +132,6 @@ for (ii in 1 : ncol(miu.rule.t)) {
 
 ## collect new parameters
 param.new <- list(func.tsk.new = func.tsk.new, varinp.mf = varinp.mf)
-
 return(param.new)
 }
 
@@ -162,17 +140,17 @@ return(param.new)
 #' the HyFIS method.
 #'
 #' @title HyFIS updating function
-#' @param data.train a matrix(m x n) of normalized data for the training process, 
-#' where m is the number of instances and 
-#' n is the number of variables; the last column is the output variable.
+#' @param data.train a matrix (\eqn{m \times n}) of normalized data for the training process, 
+#' where \eqn{m} is the number of instances and 
+#' \eqn{n} is the number of variables; the last column is the output variable.
 #' @param def matrix of defuzzification results. See \code{\link{defuzzifier}}.
 #' @param rule fuzzy IF-THEN rules. See \code{\link{rulebase}}.
 #' @param names.varoutput a list of names of the output variable. 
 #' @param var.mf a matrix of parameters of the membership functions. 
 #' Please see \code{\link{fuzzifier}}.
 #' @param miu.rule a matrix of degree of rules which is a result of the \code{\link{inference}}.
-#' @param num.labels a matrix(1 x n) whose elements represent 
-#' the number of labels (or fuzzy terms), where n is the number of variables.
+#' @param num.labels a matrix (\eqn{1 \times n}) whose elements represent 
+#' the number of labels (or linguistic terms), where n is the number of variables.
 #' @param MF a matrix of parameters of the membership functions 
 #' which is a result of the \code{\link{fuzzifier}}.
 #' @param step.size a real number, the step size of the gradient descent. 
@@ -181,8 +159,6 @@ return(param.new)
 # @return a matrix of parameters of membership of function
 # @export
 HyFIS.update <- function(data.train, def, rule, names.varoutput, var.mf, miu.rule, num.labels, MF, step.size= 0.001, degree.rule){
-
-
 rule.temp <- rule
 
 indx <- length(num.labels)
@@ -197,9 +173,7 @@ varinp.mf <- var.mf[, 1 : l.output, drop = FALSE]
 varout.mf <- var.mf[, (l.output + 1) : ncol(var.mf), drop = FALSE]
 
 ##Check not zero on miu.rule
-chck <- which(miu.rule[1, ] > 0.001)
-
-
+chck <- which(miu.rule[1, ] > 0.0001)
 l.chck <- length(chck)
 			
 ## initialize
@@ -216,8 +190,9 @@ if (l.chck != 0) {
 	## along number of not zero on miu.rule, check and save string related on names.varoutput and its value
 	for (ii in 1 : l.chck){
 		#get the names of output value on each rule
-		strg <- c(rule.temp[chck[ii], ncol(rule.temp)])
+		strg <- c(rule.temp[chck[ii], ncol(rule.temp)])		
 		aaa <- which(strg == names.varoutput)
+		
 		if (length(aaa) != 0) {
 			indx[ii, 1] <- aaa
 			indx[ii, 2] <- miu.rule[1, chck[ii]]
@@ -225,25 +200,13 @@ if (l.chck != 0) {
 		}
 	}			
 	
-## check duplication on indx, choose with has a max of degree
-if (nrow(indx) >= 2){
-	for (jj in 1 : (nrow(indx) - 1)){
-		temp.indx[1, ] <- indx[jj, ]
-		for (kk in (jj + 1) : nrow(indx)){
-			ck <- length(which(temp.indx[1, 1] == indx[kk, 1]))
-			if (ck != 0){
-				if ((temp.indx[1, 2] * temp.indx[1, 3]) >= (indx[kk, 2] * indx[kk, 3]))
-					indx[kk, ] <- NA
-				else 
-					indx[jj, ] <- NA
-				}
-			}
-		}				
-	}
-
-# erase NA value on indx
-indx <- na.omit(indx)
-
+	## check duplication on indx, choose with has a max of degree
+	indx.temp <- cbind(indx, (indx[, 2] * indx[, 3]))
+	indx.temp <- indx.temp[order(indx.temp[,c(4)], decreasing = TRUE),]
+	indx.nondup <- which(duplicated(indx.temp[, 1]) == FALSE, arr.ind = TRUE)
+	indx.temp <- indx.temp[indx.nondup, ,drop = FALSE] 
+	indx <- indx.temp[, -4]
+	
 	#####################
 	## Update center(mean) and width(variance) on output variable (layer 5)
 	####################
@@ -274,8 +237,7 @@ indx <- na.omit(indx)
 			temp <- delta * indx.comp[i, 2] * (varout.mf[2, i] * sum(indx.comp[, 2] * var.t) - sum(indx.comp[, 2] * mean.t * var.t))* (1 / (sum(indx.comp[, 2] * var.t))^2)
 			varout.mf[3, i] <- varout.mf[3, i] + eta.o * temp
 			varout.mf[2, i] <- varout.mf[2, i] + eta.o * delta * (varout.mf[2, i] * indx.comp[i, 2]) / (sum(var.t * indx.comp[, 2]))
-		}
-		else {
+		} else {
 			delta.k4 <-  0
 		}
 			
